"""Config 模块."""
from .config import (
    get_connection_safety,
    load_connections,
    load_profile,
    save_profile,
    load_safety,
    update_profile_connection_version,
    update_profile_last_connection,
    get_cached_es_version,
    PROFILE_FILE,
)

__all__ = [
    "get_connection_safety",
    "load_connections",
    "load_profile",
    "save_profile",
    "load_safety",
    "update_profile_connection_version",
    "update_profile_last_connection",
    "get_cached_es_version",
    "PROFILE_FILE",
]



"""ES 操作工具."""
from __future__ import annotations

from typing import Any, Optional

from .. import client as es_client
from .connections import get_es_client, get_es_version, get_safety


def _check_read_only() -> None:
    """检查是否为只读模式."""
    safety = get_safety()
    if safety.read_only:
        raise ValueError("当前为只读模式，拒绝执行写操作")


def _check_delete_index() -> None:
    """检查是否允许删除索引."""
    safety = get_safety()
    if not safety.allow_delete_index:
        raise ValueError("当前环境不允许删除索引")


def _check_create_index() -> None:
    """检查是否允许创建索引."""
    safety = get_safety()
    if not safety.allow_create_index:
        raise ValueError("当前环境不允许创建索引")


def es_search(
    index: str,
    query: dict | None = None,
    size: int = 10,
    from_: int = 0,
) -> dict:
    """搜索文档 (from/size 分页)."""
    client = get_es_client()
    es_version = get_es_version()
    safety = get_safety()

    # 限制大小
    if size > safety.max_search_size:
        size = safety.max_search_size

    result = es_client.es_search(
        client=client,
        es_version=es_version,
        index=index,
        query=query,
        size=size,
        from_=from_,
    )

    return {
        "hits": result.hits,
        "total": result.total,
        "truncated": result.truncated,
    }


def es_scroll(
    index: str,
    query: dict | None = None,
    size: int = 100,
    scroll: str = "5m",
) -> dict:
    """滚动查询 (首次调用)."""
    client = get_es_client()
    es_version = get_es_version()

    result = es_client.es_scroll(
        client=client,
        es_version=es_version,
        index=index,
        query=query,
        size=size,
        scroll=scroll,
    )

    return {
        "hits": result.hits,
        "scroll_id": result.scroll_id,
        "took_ms": result.took_ms,
    }


def es_scroll_next(scroll_id: str, scroll: str = "5m") -> dict:
    """滚动查询下一页."""
    client = get_es_client()

    result = es_client.es_scroll_next(
        client=client,
        scroll_id=scroll_id,
        scroll=scroll,
    )

    return {
        "hits": result.hits,
        "scroll_id": result.scroll_id,
        "took_ms": result.took_ms,
    }


def es_scroll_clear(scroll_id: str) -> dict:
    """清除滚动上下文."""
    client = get_es_client()
    es_client.es_scroll_clear(client=client, scroll_id=scroll_id)
    return {"success": True, "message": "滚动上下文已清除"}


def es_index_doc(
    index: str,
    doc: dict,
    id_: str | None = None,
) -> dict:
    """索引文档."""
    _check_read_only()
    client = get_es_client()
    es_version = get_es_version()

    doc_id = es_client.es_index_doc(
        client=client,
        es_version=es_version,
        index=index,
        doc=doc,
        id_=id_,
    )

    return {"success": True, "_id": doc_id}


def es_update_doc(
    index: str,
    id_: str,
    doc: dict,
) -> dict:
    """更新文档."""
    _check_read_only()
    client = get_es_client()
    es_version = get_es_version()

    es_client.es_update_doc(
        client=client,
        es_version=es_version,
        index=index,
        id_=id_,
        doc=doc,
    )

    return {"success": True, "message": f"文档 {id_} 已更新"}


def es_delete_doc(
    index: str,
    id_: str,
) -> dict:
    """删除文档."""
    _check_read_only()
    client = get_es_client()
    es_version = get_es_version()

    es_client.es_delete_doc(
        client=client,
        es_version=es_version,
        index=index,
        id_=id_,
    )

    return {"success": True, "message": f"文档 {id_} 已删除"}


def es_bulk(
    operations: list[dict],
    refresh: bool = False,
) -> dict:
    """批量操作."""
    _check_read_only()
    client = get_es_client()
    es_version = get_es_version()

    result = es_client.es_bulk(
        client=client,
        es_version=es_version,
        operations=operations,
        refresh=refresh,
    )

    return {
        "succeeded": result.succeeded,
        "failed": result.failed,
        "errors": result.errors,
    }


def es_create_index(
    index: str,
    mappings: dict | None = None,
    settings: dict | None = None,
) -> dict:
    """创建索引."""
    _check_read_only()
    _check_create_index()
    client = get_es_client()
    es_version = get_es_version()

    es_client.es_create_index(
        client=client,
        es_version=es_version,
        index=index,
        mappings=mappings,
        settings=settings,
    )

    return {"success": True, "message": f"索引 {index} 已创建"}


def es_delete_index(index: str) -> dict:
    """删除索引."""
    _check_read_only()
    _check_delete_index()
    client = get_es_client()

    success = es_client.es_delete_index(client=client, index=index)

    if success:
        return {"success": True, "message": f"索引 {index} 已删除"}
    else:
        return {"success": False, "message": f"索引 {index} 不存在"}


def es_index_exists(index: str) -> dict:
    """检查索引是否存在."""
    client = get_es_client()

    exists = es_client.es_index_exists(client=client, index=index)

    return {"exists": exists, "index": index}


def es_get_index_mapping(index: str) -> dict:
    """获取索引 mapping."""
    client = get_es_client()
    es_version = get_es_version()

    mapping = es_client.es_get_index_mapping(
        client=client,
        es_version=es_version,
        index=index,
    )

    return {"mapping": mapping}


def es_list_indices() -> dict:
    """列出所有索引."""
    client = get_es_client()
    es_version = get_es_version()

    indices = es_client.es_list_indices(client=client, es_version=es_version)

    return {"indices": indices, "count": len(indices)}



"""ES 连接封装."""
from __future__ import annotations

from elasticsearch import Elasticsearch
from elasticsearch.exceptions import NotFoundError

from .models import (
    BulkResult,
    ConnectionConfig,
    EsVersionInfo,
    SafetyConfig,
    ScrollResult,
    SearchResult,
)


def create_client(config: ConnectionConfig, es_version: int = 7) -> Elasticsearch:
    """创建 ES 客户端."""
    kwargs = {
        "hosts": [config.url],
        "verify_certs": False,
    }

    if config.username and config.password:
        # ES 8 使用 basic_auth，ES 6/7 使用 http_auth
        if es_version >= 8:
            kwargs["basic_auth"] = (config.username, config.password)
        else:
            kwargs["http_auth"] = (config.username, config.password)

    return Elasticsearch(**kwargs)


def get_es_version(client: Elasticsearch) -> EsVersionInfo:
    """获取 ES 版本信息."""
    info = client.info()
    version_str = info["version"]["number"]
    parts = version_str.split(".")
    return EsVersionInfo(
        major=int(parts[0]),
        minor=int(parts[1]),
        patch=int(parts[2].split("-")[0]) if "-" in parts[2] else int(parts[2]),
    )


def es_search(
    client: Elasticsearch,
    es_version: int,
    index: str,
    query: dict | None = None,
    size: int = 10,
    from_: int = 0,
) -> SearchResult:
    """搜索文档 (from/size 分页)."""
    body = query or {"query": {"match_all": {}}}

    # ES 8+ 不支持 _type
    if es_version >= 8:
        body = {k: v for k, v in body.items() if k != "_type"}

    # 移除 _type 字段
    if "_type" in body:
        del body["_type"]

    # ES 6 需要 doc_type 参数
    kwargs = {"body": body, "size": size, "from_": from_}
    if es_version < 7:
        kwargs["doc_type"] = "_doc"

    resp = client.search(index=index, **kwargs)

    hits = [hit["_source"] for hit in resp["hits"]["hits"]]
    # ES 6 返回 total 是数字，ES 7+ 返回 {"value": n}
    total = resp["hits"]["total"]
    if isinstance(total, dict):
        total = total["value"]

    return SearchResult(
        hits=hits,
        total=total,
        truncated=total > size,
    )


def es_scroll(
    client: Elasticsearch,
    es_version: int,
    index: str,
    query: dict | None = None,
    size: int = 100,
    scroll: str = "5m",
) -> ScrollResult:
    """滚动查询 (首次调用)."""
    body = query or {"query": {"match_all": {}}}

    # ES 8+ 移除 _type
    if es_version >= 8:
        body = {k: v for k, v in body.items() if k != "_type"}

    # ES 6 需要 doc_type 参数
    kwargs = {"body": body, "size": size, "scroll": scroll}
    if es_version < 7:
        kwargs["doc_type"] = "_doc"

    resp = client.search(index=index, **kwargs)

    hits = [hit["_source"] for hit in resp["hits"]["hits"]]
    scroll_id = resp.get("_scroll_id")

    return ScrollResult(
        hits=hits,
        scroll_id=scroll_id,
        took_ms=resp.get("took", 0),
    )


def es_scroll_next(
    client: Elasticsearch,
    scroll_id: str,
    scroll: str = "5m",
) -> ScrollResult:
    """滚动查询下一页."""
    resp = client.scroll(scroll_id=scroll_id, scroll=scroll)

    hits = [hit["_source"] for hit in resp["hits"]["hits"]]
    new_scroll_id = resp.get("_scroll_id")

    return ScrollResult(
        hits=hits,
        scroll_id=new_scroll_id,
        took_ms=resp.get("took", 0),
    )


def es_scroll_clear(client: Elasticsearch, scroll_id: str) -> None:
    """清除滚动上下文."""
    client.clear_scroll(scroll_id=scroll_id)


def es_index_doc(
    client: Elasticsearch,
    es_version: int,
    index: str,
    doc: dict,
    id_: str | None = None,
) -> str:
    """索引文档."""
    kwargs = {"body": doc}

    if es_version >= 8:
        kwargs["index"] = index
        if id_:
            result = client.index(index=index, id=id_, body=doc)
        else:
            result = client.index(index=index, body=doc)
    else:
        kwargs["doc_type"] = "_doc"
        if id_:
            result = client.index(index=index, doc_type="_doc", id=id_, body=doc)
        else:
            result = client.index(index=index, doc_type="_doc", body=doc)

    return result["_id"]


def es_update_doc(
    client: Elasticsearch,
    es_version: int,
    index: str,
    id_: str,
    doc: dict,
) -> None:
    """更新文档."""
    if es_version >= 8:
        client.update(index=index, id=id_, body={"doc": doc})
    else:
        client.update(index=index, doc_type="_doc", id=id_, body={"doc": doc})


def es_delete_doc(
    client: Elasticsearch,
    es_version: int,
    index: str,
    id_: str,
) -> None:
    """删除文档."""
    if es_version >= 8:
        client.delete(index=index, id=id_)
    else:
        client.delete(index=index, doc_type="_doc", id=id_)


def es_bulk(
    client: Elasticsearch,
    es_version: int,
    operations: list[dict],
    refresh: bool = False,
) -> BulkResult:
    """批量操作."""
    if es_version >= 8:
        resp = client.bulk(body=operations, refresh=refresh)
    else:
        resp = client.bulk(body=operations, refresh=refresh)

    succeeded = 0
    failed = 0
    errors = []

    for item in resp.get("items", []):
        action = item.get("index") or item.get("update") or item.get("delete") or item.get("create")
        if action.get("error"):
            failed += 1
            errors.append({
                "action": list(item.keys())[0],
                "id": action.get("_id"),
                "error": action["error"].get("reason", "Unknown error"),
            })
        else:
            succeeded += 1

    return BulkResult(succeeded=succeeded, failed=failed, errors=errors)


def es_create_index(
    client: Elasticsearch,
    es_version: int,
    index: str,
    mappings: dict | None = None,
    settings: dict | None = None,
) -> bool:
    """创建索引."""
    body = {}
    if mappings:
        body["mappings"] = mappings
    if settings:
        body["settings"] = settings

    if es_version >= 8:
        return client.indices.create(index=index, body=body if body else None)
    else:
        return client.indices.create(index=index, doc_type="_doc", body=body if body else None)


def es_delete_index(client: Elasticsearch, index: str) -> bool:
    """删除索引."""
    try:
        client.indices.delete(index=index)
        return True
    except NotFoundError:
        return False


def es_index_exists(client: Elasticsearch, index: str) -> bool:
    """检查索引是否存在."""
    return client.indices.exists(index=index)


def es_get_index_mapping(
    client: Elasticsearch,
    es_version: int,
    index: str,
) -> dict:
    """获取索引 mapping."""
    if es_version >= 8:
        return client.indices.get_mapping(index=index)
    else:
        return client.indices.get_mapping(index=index, doc_type="_doc")


def es_list_indices(client: Elasticsearch, es_version: int = 7) -> list[dict]:
    """列出所有索引."""
    resp = client.cat.indices(format="json")
    indices = []
    for idx in resp:
        # ES 6 字段名可能不同，尝试多种可能的字段名
        name = idx.get("index") or idx.get("uuid")  # ES 6 可能用 uuid
        if not name:
            continue

        # 处理 docs.count 字段名差异
        docs_count = idx.get("docs.count") or idx.get("docs.count") or idx.get("documents.count") or 0
        docs_count = int(docs_count) if docs_count else 0

        # 处理 store.size 字段名差异
        size = idx.get("store.size") or idx.get("store.size") or idx.get("disk.usage") or "0b"

        indices.append({
            "name": name,
            "docs_count": docs_count,
            "size": size,
            "health": idx.get("health", "unknown"),
        })
    return indices



"""Resources 模块."""
from ..tools.connections import get_current_connection, get_session_state


def get_es_connections_resource() -> dict:
    """获取连接列表资源."""
    from ..config import load_connections

    connections = load_connections()
    return {
        "connections": [
            {
                "name": conn.name,
                "env": conn.env,
                "description": conn.description,
            }
            for conn in connections
        ]
    }


def get_es_current_resource() -> dict:
    """获取当前连接资源."""
    state = get_session_state()
    current = get_current_connection()

    if not current:
        return {"current": None, "message": "当前没有选中的连接"}

    return {
        "current": current,
        "env": state.env,
        "es_version": state.es_version,
        "es_version_info": (
            f"{state.es_version_info.major}.{state.es_version_info.minor}.{state.es_version_info.patch}"
            if state.es_version_info else None
        ),
        "safety": {
            "read_only": state.safety.read_only if state.safety else None,
            "max_search_size": state.safety.max_search_size if state.safety else None,
        },
    }


def get_es_status_resource() -> dict:
    """获取 ES MCP 状态资源."""
    from ..config import PROFILE_FILE

    state = get_session_state()

    return {
        "status": "running" if state.current_connection else "idle",
        "profile_file": str(PROFILE_FILE),
        "config_source": "ES_CONNECTIONS environment variable",
        "current_connection": state.current_connection,
    }



"""连接管理工具."""
from __future__ import annotations

from typing import Optional

from .. import client as es_client
from ..config import (
    get_cached_es_version,
    load_connections,
    load_profile,
    load_safety,
    update_profile_connection_version,
    update_profile_last_connection,
)
from ..models import (
    ConnectionConfig,
    ConnectionItem,
    EsVersionInfo,
    SafetyConfig,
    SessionState,
)

# 全局会话状态
_session_state: SessionState = SessionState()


def get_session_state() -> SessionState:
    """获取会话状态."""
    return _session_state


def get_current_connection_config() -> ConnectionConfig:
    """获取当前连接配置."""
    if not _session_state.current_connection:
        raise ValueError("当前没有选中的连接，请先调用 select_connection")

    connections = load_connections()
    for conn in connections:
        if conn.name == _session_state.current_connection:
            return conn

    raise ValueError(f"未找到连接: {_session_state.current_connection}")


def list_connections() -> list[ConnectionItem]:
    """列出所有配置的连接."""
    connections = load_connections()
    return [
        ConnectionItem(
            name=conn.name,
            env=conn.env,
            description=conn.description,
        )
        for conn in connections
    ]


def get_current_connection() -> Optional[str]:
    """获取当前连接名."""
    return _session_state.current_connection


def select_connection(name: str) -> dict:
    """
    选择连接并获取版本信息.
    返回连接信息和版本
    """
    global _session_state

    # 查找连接配置
    connections = load_connections()
    conn_config = None
    for conn in connections:
        if conn.name == name:
            conn_config = conn
            break

    if not conn_config:
        available = [c.name for c in connections]
        raise ValueError(f"未找到连接: {name}，可用: {', '.join(available)}")

    # 创建 ES 客户端
    es_client_conn = es_client.create_client(conn_config)

    # 检查是否有缓存版本
    cached_version = get_cached_es_version(name)

    if cached_version:
        es_version_info = cached_version
    else:
        # 连接并获取版本
        es_version_info = es_client.get_es_version(es_client_conn)
        # 缓存版本
        update_profile_connection_version(name, es_version_info)

    # 加载 safety 配置
    safety = load_safety(conn_config.env)

    # 更新会话状态
    _session_state = SessionState(
        current_connection=name,
        env=conn_config.env,
        es_version=es_version_info.version,
        es_version_info=es_version_info,
        safety=safety,
    )

    # 保存最后使用的连接
    update_profile_last_connection(name)

    return {
        "name": name,
        "env": conn_config.env,
        "description": conn_config.description,
        "es_version": es_version_info.version,
        "es_version_info": f"{es_version_info.major}.{es_version_info.minor}.{es_version_info.patch}",
        "safety": {
            "read_only": safety.read_only,
            "max_search_size": safety.max_search_size,
            "confirm_delete": safety.confirm_delete,
            "allow_delete_index": safety.allow_delete_index,
            "allow_create_index": safety.allow_create_index,
            "risk_check": safety.risk_check,
        },
    }


def get_es_client() -> es_client.Elasticsearch:
    """获取当前 ES 客户端."""
    config = get_current_connection_config()
    es_version = get_es_version()
    return es_client.create_client(config, es_version=es_version)


def get_es_version() -> int:
    """获取当前 ES 版本."""
    if not _session_state.es_version:
        raise ValueError("当前没有选中的连接")
    return _session_state.es_version


def get_safety() -> SafetyConfig:
    """获取当前安全配置."""
    if not _session_state.safety:
        raise ValueError("当前没有选中的连接")
    return _session_state.safety


def auto_select_connection() -> Optional[dict]:
    """
    自动选择上次使用的连接.
    用于启动时恢复会话.
    如果上次连接为空或不在列表中，使用第一个连接.
    """
    profile = load_profile()
    connections = load_connections()

    # 确定要连接的名称
    target_name = profile.last_connection

    # 如果上次连接为空或不在列表中，使用第一个连接
    if not target_name or target_name not in [c.name for c in connections]:
        if connections:
            target_name = connections[0].name
        else:
            return None

    try:
        return select_connection(target_name)
    except Exception:
        # 如果选中的连接失败，尝试第一个
        if connections and target_name != connections[0].name:
            try:
                return select_connection(connections[0].name)
            except Exception:
                pass
        return None



"""配置加载."""
from __future__ import annotations

import json
import os
import sys
from pathlib import Path
from typing import Optional

from ..models import ConnectionConfig, EsVersionInfo, ProfileData, SafetyConfig


# ============== 路径配置 ==============

def _get_config_dir() -> Path:
    """获取配置目录."""
    xdg = os.environ.get("XDG_CONFIG_HOME")
    if xdg:
        return Path(xdg) / "elasticsearch-client-mcp"
    return Path.home() / ".config" / "elasticsearch-client-mcp"


PROFILE_FILE = _get_config_dir() / "profile.json"

# 默认 profile.json 模板路径（用于创建初始配置）
DEFAULT_PROFILE_TEMPLATE = Path(__file__).parent.parent.parent / "config" / "profile_template.json"


# ============== 加载连接配置（从环境变量） ==============

ES_CONNECTIONS_HELP = """
ES_CONNECTIONS 配置模板 (添加到 ~/.claude/settings.json 的 mcpServers 中):

{
  "mcpServers": {
    "elasticsearch-client": {
      "command": "uvx",
      "args": ["elasticsearch-client-mcp-py"],
      "env": {
        "ES_CONNECTIONS": "[{\\"name\\":\\"your_name\\",\\"env\\":\\"prod\\",\\"url\\":\\"http://your-es:9200\\",\\"username\\":\\"elastic\\",\\"password\\":\\"your_password\\"}]"
      },
      "type": "stdio"
    }
  }
}
"""


def load_connections() -> list[ConnectionConfig]:
    """从 ES_CONNECTIONS 环境变量加载连接配置."""
    env_connections = os.environ.get("ES_CONNECTIONS", "")

    if not env_connections:
        print("错误: ES_CONNECTIONS 环境变量未设置", file=sys.stderr)
        print(ES_CONNECTIONS_HELP, file=sys.stderr)
        print("权限配置: ~/.config/elasticsearch-client-mcp/profile.json", file=sys.stderr)
        sys.exit(1)

    try:
        connections_data = json.loads(env_connections)
    except json.JSONDecodeError as e:
        print(f"错误: ES_CONNECTIONS JSON 解析失败: {e}", file=sys.stderr)
        print(ES_CONNECTIONS_HELP, file=sys.stderr)
        sys.exit(1)

    if not connections_data:
        print("错误: ES_CONNECTIONS 中没有配置任何连接", file=sys.stderr)
        print(ES_CONNECTIONS_HELP, file=sys.stderr)
        sys.exit(1)

    connections = []
    for item in connections_data:
        # 安全配置：优先使用内联的 safety，否则为 None（后续从 profile 加载）
        safety = None
        if "safety" in item:
            safety_data = item["safety"]
            safety = SafetyConfig(
                read_only=safety_data.get("read_only", False),
                max_search_size=safety_data.get("max_search_size", 100),
                confirm_delete=safety_data.get("confirm_delete", True),
                allow_delete_index=safety_data.get("allow_delete_index", False),
                allow_create_index=safety_data.get("allow_create_index", False),
                risk_check=safety_data.get("risk_check", True),
            )

        conn = ConnectionConfig(
            name=str(item["name"]),
            env=str(item["env"]),
            description=item.get("description"),
            url=str(item.get("url", "")),
            username=str(item.get("username", "")),
            password=str(item.get("password", "")),
        )
        connections.append(conn)

    return connections


def get_connection_safety(conn: ConnectionConfig) -> SafetyConfig:
    """获取连接的安全配置.

    优先级：
    1. 连接内联的 safety（已解析）
    2. profile.json 中对应 env 的 safety
    """
    # 先检查 profile 中的 safety
    profile = load_profile()
    env_data = profile.envs.get(conn.env, {})
    safety_data = env_data.get("safety", {})

    return SafetyConfig(
        read_only=safety_data.get("read_only", False),
        max_search_size=safety_data.get("max_search_size", 100),
        confirm_delete=safety_data.get("confirm_delete", True),
        allow_delete_index=safety_data.get("allow_delete_index", False),
        allow_create_index=safety_data.get("allow_create_index", False),
        risk_check=safety_data.get("risk_check", True),
    )


# ============== Profile 操作 ==============

def load_profile() -> ProfileData:
    """从 profile.json 加载 profile 数据."""
    if not PROFILE_FILE.exists():
        # 如果 profile.json 不存在，尝试从模板创建
        if DEFAULT_PROFILE_TEMPLATE.exists():
            _get_config_dir().mkdir(parents=True, exist_ok=True)
            with open(DEFAULT_PROFILE_TEMPLATE, "r", encoding="utf-8") as f:
                template_data = json.load(f)
            with open(PROFILE_FILE, "w", encoding="utf-8") as f:
                json.dump(template_data, f, indent=2, ensure_ascii=False)
        else:
            return ProfileData()

    with open(PROFILE_FILE, "r", encoding="utf-8") as f:
        data = json.load(f)

    profile = ProfileData()
    profile.last_connection = data.get("_last_connection")

    # 加载连接缓存的版本信息
    for name, conn_data in data.get("_connection_cache", {}).items():
        profile.connections[name] = conn_data

    # 加载环境配置（safety 等）
    for env_name, env_data in data.get("_envs", {}).items():
        profile.envs[env_name] = env_data

    return profile


def save_profile(profile: ProfileData) -> None:
    """保存 profile 数据到 profile.json."""
    _get_config_dir().mkdir(parents=True, exist_ok=True)

    data = {
        "_last_connection": profile.last_connection,
        "_connection_cache": profile.connections,
        "_envs": profile.envs,
    }

    with open(PROFILE_FILE, "w", encoding="utf-8") as f:
        json.dump(data, f, indent=2, ensure_ascii=False)


def load_safety(env: str) -> SafetyConfig:
    """加载指定环境的安全配置."""
    profile = load_profile()

    env_data = profile.envs.get(env, {})
    safety_data = env_data.get("safety", {})

    return SafetyConfig(
        read_only=safety_data.get("read_only", False),
        max_search_size=safety_data.get("max_search_size", 100),
        confirm_delete=safety_data.get("confirm_delete", True),
        allow_delete_index=safety_data.get("allow_delete_index", False),
        allow_create_index=safety_data.get("allow_create_index", False),
        risk_check=safety_data.get("risk_check", True),
    )


def update_profile_connection_version(connection_name: str, es_version_info: EsVersionInfo) -> None:
    """更新连接的版本缓存."""
    profile = load_profile()

    if connection_name not in profile.connections:
        profile.connections[connection_name] = {}

    profile.connections[connection_name] = {
        "es_version": es_version_info.major,
        "es_version_info": [es_version_info.major, es_version_info.minor, es_version_info.patch],
    }

    save_profile(profile)


def update_profile_last_connection(connection_name: str) -> None:
    """更新上次使用的连接."""
    profile = load_profile()
    profile.last_connection = connection_name
    save_profile(profile)


def get_cached_es_version(connection_name: str) -> Optional[EsVersionInfo]:
    """获取缓存的 ES 版本信息."""
    profile = load_profile()
    conn_data = profile.connections.get(connection_name, {})
    version_info = conn_data.get("es_version_info")
    if version_info and len(version_info) >= 3:
        return EsVersionInfo(
            major=version_info[0],
            minor=version_info[1],
            patch=version_info[2],
        )
    return None



"""Elasticsearch MCP Server."""
from __future__ import annotations

import sys
from typing import Any

from mcp.server.fastmcp import FastMCP

from .config import load_connections
from .tools.connections import (
    auto_select_connection,
    get_current_connection as _get_current_connection,
    get_session_state,
    list_connections as _list_connections,
    select_connection as _select_connection,
)
from .tools import es_operations
from .resources.config import (
    get_es_connections_resource,
    get_es_current_resource,
    get_es_status_resource,
)


# ============== MCP Server ==============

mcp = FastMCP("Elasticsearch")


# ============== 数据模型 ==============

class ConnectionListResult:
    """连接列表结果."""
    def __init__(self, connections: list, default: str | None):
        self.connections = connections
        self.default = default


# ============== Tools ==============

@mcp.tool()
def list_connections() -> ConnectionListResult:
    """列出所有已配置的 ES 连接."""
    connections = _list_connections()
    session = get_session_state()
    default = session.current_connection

    items = [
        {
            "name": conn.name,
            "env": conn.env,
            "description": conn.description,
        }
        for conn in connections
    ]

    return ConnectionListResult(connections=items, default=default)


@mcp.tool()
def get_current_connection() -> dict:
    """获取当前选中的 ES 连接."""
    current = _get_current_connection()
    if not current:
        return {"current": None, "message": "当前没有选中的连接，请先调用 select_connection"}

    session = get_session_state()
    return {
        "current": current,
        "env": session.env,
        "es_version": session.es_version,
        "safety": {
            "read_only": session.safety.read_only if session.safety else None,
            "max_search_size": session.safety.max_search_size if session.safety else None,
        },
    }


@mcp.tool()
def select_connection(name: str) -> dict:
    """选择当前会话使用的 ES 连接."""
    result = _select_connection(name)
    return {
        "message": f"已切换到连接: {name}",
        **result,
    }


@mcp.tool()
def es_search(
    index: str,
    query: dict | None = None,
    size: int = 10,
    from_: int = 0,
) -> dict:
    """搜索 ES 文档 (from/size 分页)."""
    return es_operations.es_search(index=index, query=query, size=size, from_=from_)


@mcp.tool()
def es_scroll(
    index: str,
    query: dict | None = None,
    size: int = 100,
    scroll: str = "5m",
) -> dict:
    """滚动查询 (首次调用返回 scroll_id)."""
    return es_operations.es_scroll(index=index, query=query, size=size, scroll=scroll)


@mcp.tool()
def es_scroll_next(scroll_id: str, scroll: str = "5m") -> dict:
    """滚动查询下一页，需传入上次返回的 scroll_id."""
    return es_operations.es_scroll_next(scroll_id=scroll_id, scroll=scroll)


@mcp.tool()
def es_scroll_clear(scroll_id: str) -> dict:
    """清除滚动上下文."""
    return es_operations.es_scroll_clear(scroll_id=scroll_id)


@mcp.tool()
def es_index_doc(
    index: str,
    doc: dict,
    id_: str | None = None,
) -> dict:
    """索引文档到 ES."""
    return es_operations.es_index_doc(index=index, doc=doc, id_=id_)


@mcp.tool()
def es_update_doc(
    index: str,
    id_: str,
    doc: dict,
) -> dict:
    """更新 ES 文档."""
    return es_operations.es_update_doc(index=index, id_=id_, doc=doc)


@mcp.tool()
def es_delete_doc(
    index: str,
    id_: str,
) -> dict:
    """删除 ES 文档."""
    return es_operations.es_delete_doc(index=index, id_=id_)


@mcp.tool()
def es_bulk(
    operations: list[dict],
    refresh: bool = False,
) -> dict:
    """批量操作 (支持 index/create/update/delete)."""
    return es_operations.es_bulk(operations=operations, refresh=refresh)


@mcp.tool()
def es_create_index(
    index: str,
    mappings: dict | None = None,
    settings: dict | None = None,
) -> dict:
    """创建 ES 索引."""
    return es_operations.es_create_index(index=index, mappings=mappings, settings=settings)


@mcp.tool()
def es_delete_index(index: str) -> dict:
    """删除 ES 索引."""
    return es_operations.es_delete_index(index=index)


@mcp.tool()
def es_index_exists(index: str) -> dict:
    """检查索引是否存在."""
    return es_operations.es_index_exists(index=index)


@mcp.tool()
def es_get_index_mapping(index: str) -> dict:
    """获取索引的 mapping."""
    return es_operations.es_get_index_mapping(index=index)


@mcp.tool()
def es_list_indices() -> dict:
    """列出所有索引."""
    return es_operations.es_list_indices()


# ============== Resources ==============

@mcp.resource("es://connections")
def resource_connections() -> str:
    """获取连接列表资源."""
    import json
    return json.dumps(get_es_connections_resource(), indent=2, ensure_ascii=False)


@mcp.resource("es://current")
def resource_current() -> str:
    """获取当前连接资源."""
    import json
    return json.dumps(get_es_current_resource(), indent=2, ensure_ascii=False)


@mcp.resource("es://status")
def resource_status() -> str:
    """获取 ES MCP 状态."""
    import json
    return json.dumps(get_es_status_resource(), indent=2, ensure_ascii=False)


# ============== 入口 ==============

CONFIG_TEMPLATE = '''
MCP 配置模板 (添加到 ~/.claude/settings.json 的 mcpServers 中):

{
  "mcpServers": {
    "elasticsearch-client": {
      "command": "uvx",
      "args": ["elasticsearch-client-mcp-py"],
      "env": {
        "ES_CONNECTIONS": "[{\\"name\\":\\"your_name\\",\\"env\\":\\"prod\\",\\"url\\":\\"http://your-es:9200\\",\\"username\\":\\"elastic\\",\\"password\\":\\"your_password\\"}]"
      },
      "type": "stdio"
    }
  }
}

参数说明:
- name: 连接名称（唯一标识）
- env: 环境 (dev/test/uat/prod)
- url: ES 访问地址
- username/password: 认证信息
- safety (可选): 内联权限覆盖

profile.json 权限配置位于: ~/.config/elasticsearch-client-mcp/profile.json
环境: dev, test, uat, prod
'''


def main() -> None:
    """主入口."""
    import sys

    # 尝试加载配置
    try:
        connections = load_connections()
    except SystemExit:
        # 配置加载失败，输出配置模板
        print(CONFIG_TEMPLATE, file=sys.stderr)
        raise

    print(f"""Elasticsearch MCP Server 已启动
========================
已配置 {len(connections)} 个连接

配置来源: ES_CONNECTIONS 环境变量
权限配置: ~/.config/elasticsearch-client-mcp/profile.json
环境: dev, test, uat, prod
""", file=sys.stderr)

    # 尝试自动恢复上次连接
    auto_result = auto_select_connection()
    if auto_result:
        print(f"已自动恢复上次连接: {auto_result['name']}", file=sys.stderr)
    else:
        print("请调用 select_connection 选择要使用的连接", file=sys.stderr)

    # 运行服务
    mcp.run()


if __name__ == "__main__":
    main()



"""数据模型定义."""
from __future__ import annotations

from dataclasses import dataclass, field
from typing import Optional


@dataclass
class ConnectionConfig:
    """单条连接配置"""
    name: str
    env: str
    description: Optional[str] = None
    url: str = ""
    username: str = ""
    password: str = ""


@dataclass
class SafetyConfig:
    """安全与行为配置（按 env）"""
    read_only: bool = False
    max_search_size: int = 100
    confirm_delete: bool = True
    allow_delete_index: bool = False
    allow_create_index: bool = False
    risk_check: bool = True


@dataclass
class EsVersionInfo:
    """ES 版本信息"""
    major: int  # 6, 7, 8
    minor: int
    patch: int

    @property
    def version(self) -> int:
        return self.major


@dataclass
class SessionState:
    """会话状态"""
    current_connection: Optional[str] = None
    env: Optional[str] = None
    es_version: Optional[int] = None
    es_version_info: Optional[EsVersionInfo] = None
    safety: Optional[SafetyConfig] = None


@dataclass
class ProfileData:
    """Profile 数据（持久化）"""
    last_connection: Optional[str] = None
    connections: dict[str, dict] = field(default_factory=dict)  # name -> {es_version, es_version_info}
    envs: dict[str, dict] = field(default_factory=dict)  # env -> {safety}


@dataclass
class ConnectionItem:
    """连接列表项"""
    name: str
    env: str
    description: Optional[str] = None


@dataclass
class SearchResult:
    """搜索结果"""
    hits: list[dict]
    total: int
    truncated: bool = False


@dataclass
class ScrollResult:
    """滚动查询结果"""
    hits: list[dict]
    scroll_id: Optional[str] = None
    took_ms: int = 0


@dataclass
class BulkResult:
    """批量操作结果"""
    succeeded: int
    failed: int
    errors: list[dict] = field(default_factory=list)
